pub struct Cli {Show 36 fields
pub cwd_wsr: bool,
pub stdout: Option<PathBuf>,
pub stderr: Option<PathBuf>,
pub run_all: RunAll,
pub gist: bool,
pub release: bool,
pub quiet: bool,
pub pre_build: bool,
pub cached: bool,
pub detached: bool,
pub scan_dir: Option<PathBuf>,
pub filter: bool,
pub version: bool,
pub tui: bool,
pub workspace: bool,
pub print_exit_code: bool,
pub print_program_name: bool,
pub print_instruction: bool,
pub paging: bool,
pub relative_numbers: bool,
pub wait: u64,
pub subcommand: String,
pub explicit_example: Option<String>,
pub run_at_a_time: usize,
pub no_status_lines: bool,
pub no_tts: bool,
pub parse_available: bool,
pub default_binary_is_runner: bool,
pub no_window: bool,
pub log: Option<PathBuf>,
pub manifest_path: Option<PathBuf>,
pub target: Option<String>,
pub json_all_targets: bool,
pub extra: Vec<String>,
pub detached_hold: Option<u32>,
pub detached_delay: Option<u32>,
}Fields
cwd_wsr: boolRun targets from the workspace root instead of the current working directory.
stdout: Option<PathBuf>Run all examples for a given number of seconds. Path to read/write the stdout of the executed command.
stderr: Option<PathBuf>Path to read/write the stderr of the executed command.
run_all: RunAllIf provided with a value (e.g. --run-all 10), each target will run for 10 seconds.
If provided without a value (i.e. just --run-all), it means run forever.
If not provided at all, then the default wait time is used.
gist: boolrelease: boolquiet: boolpre_build: boolPre-build examples before running.
cached: booldetached: boolScan the given directory for targets to run. Run the command in detached mode.
scan_dir: Option<PathBuf>filter: boolEnable filter mode. cargo output is filtered and captured.“
version: boolPrint version and feature flags in JSON format.
tui: boolworkspace: boolprint_exit_code: boolPrint the exit code of the process when run.
print_program_name: boolPrint the program name before execution.
print_instruction: boolPrint the program name before execution.
paging: boolrelative_numbers: boolwait: u64subcommand: StringSubcommands to run (e.g., build|b, test|t).
explicit_example: Option<String>run_at_a_time: usizeno_status_lines: boolno_tts: boolparse_available: boolParse available targets from stdin (one per line).
default_binary_is_runner: boolno_window: boollog: Option<PathBuf>Enable logging to a file or stdout.
manifest_path: Option<PathBuf>target: Option<String>json_all_targets: boolOutput the list of targets as JSON.
extra: Vec<String>detached_hold: Option<u32>detached_delay: Option<u32>Trait Implementations
Sourceimpl Args for Cli
impl Args for Cli
Sourcefn augment_args<'b>(__clap_app: Command) -> Command
fn augment_args<'b>(__clap_app: Command) -> Command
Sourcefn augment_args_for_update<'b>(__clap_app: Command) -> Command
fn augment_args_for_update<'b>(__clap_app: Command) -> Command
Command so it can instantiate self via
FromArgMatches::update_from_arg_matches_mut Read moreSourceimpl CommandFactory for Cli
impl CommandFactory for Cli
Sourceimpl FromArgMatches for Cli
impl FromArgMatches for Cli
Sourcefn from_arg_matches(__clap_arg_matches: &ArgMatches) -> Result<Self, Error>
fn from_arg_matches(__clap_arg_matches: &ArgMatches) -> Result<Self, Error>
Sourcefn from_arg_matches_mut(
__clap_arg_matches: &mut ArgMatches,
) -> Result<Self, Error>
fn from_arg_matches_mut( __clap_arg_matches: &mut ArgMatches, ) -> Result<Self, Error>
Sourcefn update_from_arg_matches(
&mut self,
__clap_arg_matches: &ArgMatches,
) -> Result<(), Error>
fn update_from_arg_matches( &mut self, __clap_arg_matches: &ArgMatches, ) -> Result<(), Error>
ArgMatches to self.Sourcefn update_from_arg_matches_mut(
&mut self,
__clap_arg_matches: &mut ArgMatches,
) -> Result<(), Error>
fn update_from_arg_matches_mut( &mut self, __clap_arg_matches: &mut ArgMatches, ) -> Result<(), Error>
ArgMatches to self.Sourceimpl Parser for Cli
impl Parser for Cli
Sourcefn parse_from<I, T>(itr: I) -> Self
fn parse_from<I, T>(itr: I) -> Self
Sourcefn try_parse_from<I, T>(itr: I) -> Result<Self, Error>
fn try_parse_from<I, T>(itr: I) -> Result<Self, Error>
Sourcefn update_from<I, T>(&mut self, itr: I)
fn update_from<I, T>(&mut self, itr: I)