SignalTimeTracker

Trait SignalTimeTracker 

pub trait SignalTimeTracker {
    // Required methods
    fn last_signal_time(&self) -> Option<SystemTime>;
    fn time_between_signals(&self) -> Option<Duration>;
}

Required Methods

fn last_signal_time(&self) -> Option<SystemTime>

Returns the time when the last signal was received, if any.

fn time_between_signals(&self) -> Option<Duration>

Returns the duration between the last two signals, if at least two signals were received.

Dyn Compatibility

This trait is dyn compatible.

In older versions of Rust, dyn compatibility was called "object safety".

Implementors