pub trait Error: Debug + Display {
// Provided methods
fn source(&self) -> Option<&(dyn Error + 'static)> { ... }
fn description(&self) -> &str { ... }
fn cause(&self) -> Option<&dyn Error> { ... }
fn provide<'a>(&'a self, request: &mut Request<'a>) { ... }
}Expand description
Error is a trait representing the basic expectations for error values,
i.e., values of type E in Result<T, E>.
Errors must describe themselves through the Display and Debug
traits. Error messages are typically concise lowercase sentences without
trailing punctuation:
let err = "NaN".parse::<u32>().unwrap_err();
assert_eq!(err.to_string(), "invalid digit found in string");Error source
Errors may provide cause information. Error::source() is generally
used when errors cross “abstraction boundaries”. If one module must report
an error that is caused by an error from a lower-level module, it can allow
accessing that error via Error::source(). This makes it possible for the
high-level module to provide its own errors while also revealing some of the
implementation for debugging.
In error types that wrap an underlying error, the underlying error
should be either returned by the outer error’s Error::source(), or rendered
by the outer error’s Display implementation, but not both.
Example
Implementing the Error trait only requires that Debug and Display are implemented too.
use std::error::Error;
use std::fmt;
use std::path::PathBuf;
#[derive(Debug)]
struct ReadConfigError {
path: PathBuf
}
impl fmt::Display for ReadConfigError {
fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
let path = self.path.display();
write!(f, "unable to read configuration at {path}")
}
}
impl Error for ReadConfigError {}Provided Methods
1.30.0 · Sourcefn source(&self) -> Option<&(dyn Error + 'static)>
fn source(&self) -> Option<&(dyn Error + 'static)>
Returns the lower-level source of this error, if any.
Examples
use std::error::Error;
use std::fmt;
#[derive(Debug)]
struct SuperError {
source: SuperErrorSideKick,
}
impl fmt::Display for SuperError {
fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
write!(f, "SuperError is here!")
}
}
impl Error for SuperError {
fn source(&self) -> Option<&(dyn Error + 'static)> {
Some(&self.source)
}
}
#[derive(Debug)]
struct SuperErrorSideKick;
impl fmt::Display for SuperErrorSideKick {
fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
write!(f, "SuperErrorSideKick is here!")
}
}
impl Error for SuperErrorSideKick {}
fn get_super_error() -> Result<(), SuperError> {
Err(SuperError { source: SuperErrorSideKick })
}
fn main() {
match get_super_error() {
Err(e) => {
println!("Error: {e}");
println!("Caused by: {}", e.source().unwrap());
}
_ => println!("No error"),
}
}1.0.0 · Sourcefn description(&self) -> &str
👎Deprecated since 1.42.0: use the Display impl or to_string()
fn description(&self) -> &str
use the Display impl or to_string()
if let Err(e) = "xc".parse::<u32>() {
// Print `e` itself, no need for description().
eprintln!("Error: {e}");
}fn cause(&self) -> Option<&dyn Error>
replaced by Error::source, which can support downcasting
Sourcefn provide<'a>(&'a self, request: &mut Request<'a>)
🔬This is a nightly-only experimental API. (error_generic_member_access)
fn provide<'a>(&'a self, request: &mut Request<'a>)
error_generic_member_access)Provides type-based access to context intended for error reports.
Used in conjunction with Request::provide_value and Request::provide_ref to extract
references to member variables from dyn Error trait objects.
Example
#![feature(error_generic_member_access)]
use core::fmt;
use core::error::{request_ref, Request};
#[derive(Debug)]
enum MyLittleTeaPot {
Empty,
}
#[derive(Debug)]
struct MyBacktrace {
// ...
}
impl MyBacktrace {
fn new() -> MyBacktrace {
// ...
}
}
#[derive(Debug)]
struct Error {
backtrace: MyBacktrace,
}
impl fmt::Display for Error {
fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
write!(f, "Example Error")
}
}
impl std::error::Error for Error {
fn provide<'a>(&'a self, request: &mut Request<'a>) {
request
.provide_ref::<MyBacktrace>(&self.backtrace);
}
}
fn main() {
let backtrace = MyBacktrace::new();
let error = Error { backtrace };
let dyn_error = &error as &dyn std::error::Error;
let backtrace_ref = request_ref::<MyBacktrace>(dyn_error).unwrap();
assert!(core::ptr::eq(&error.backtrace, backtrace_ref));
assert!(request_ref::<MyLittleTeaPot>(dyn_error).is_none());
}Delegating Impls
Warning: We recommend implementors avoid delegating implementations of provide to
source error implementations.
This method should expose context from the current piece of the source chain only, not from
sources that are exposed in the chain of sources. Delegating provide implementations cause
the same context to be provided by multiple errors in the chain of sources which can cause
unintended duplication of information in error reports or require heuristics to deduplicate.
In other words, the following implementation pattern for provide is discouraged and should
not be used for Error types exposed in public APIs to third parties.
struct MyError {
source: Error,
}
impl std::error::Error for MyError {
fn source(&self) -> Option<&(dyn std::error::Error + 'static)> {
Some(&self.source)
}
fn provide<'a>(&'a self, request: &mut Request<'a>) {
self.source.provide(request) // <--- Discouraged
}
}Implementations
Sourceimpl dyn Error
impl dyn Error
1.3.0 · Sourcepub fn is<T>(&self) -> boolwhere
T: Error + 'static,
pub fn is<T>(&self) -> boolwhere
T: Error + 'static,
Returns true if the inner type is the same as T.
1.3.0 · Sourcepub fn downcast_ref<T>(&self) -> Option<&T>where
T: Error + 'static,
pub fn downcast_ref<T>(&self) -> Option<&T>where
T: Error + 'static,
Returns some reference to the inner value if it is of type T, or
None if it isn’t.
1.3.0 · Sourcepub fn downcast_mut<T>(&mut self) -> Option<&mut T>where
T: Error + 'static,
pub fn downcast_mut<T>(&mut self) -> Option<&mut T>where
T: Error + 'static,
Returns some mutable reference to the inner value if it is of type T, or
None if it isn’t.
Sourceimpl dyn Error + Send
impl dyn Error + Send
1.3.0 · Sourcepub fn is<T>(&self) -> boolwhere
T: Error + 'static,
pub fn is<T>(&self) -> boolwhere
T: Error + 'static,
Forwards to the method defined on the type dyn Error.
1.3.0 · Sourcepub fn downcast_ref<T>(&self) -> Option<&T>where
T: Error + 'static,
pub fn downcast_ref<T>(&self) -> Option<&T>where
T: Error + 'static,
Forwards to the method defined on the type dyn Error.
1.3.0 · Sourcepub fn downcast_mut<T>(&mut self) -> Option<&mut T>where
T: Error + 'static,
pub fn downcast_mut<T>(&mut self) -> Option<&mut T>where
T: Error + 'static,
Forwards to the method defined on the type dyn Error.
Sourceimpl dyn Error + Sync + Send
impl dyn Error + Sync + Send
1.3.0 · Sourcepub fn is<T>(&self) -> boolwhere
T: Error + 'static,
pub fn is<T>(&self) -> boolwhere
T: Error + 'static,
Forwards to the method defined on the type dyn Error.
1.3.0 · Sourcepub fn downcast_ref<T>(&self) -> Option<&T>where
T: Error + 'static,
pub fn downcast_ref<T>(&self) -> Option<&T>where
T: Error + 'static,
Forwards to the method defined on the type dyn Error.
1.3.0 · Sourcepub fn downcast_mut<T>(&mut self) -> Option<&mut T>where
T: Error + 'static,
pub fn downcast_mut<T>(&mut self) -> Option<&mut T>where
T: Error + 'static,
Forwards to the method defined on the type dyn Error.
Sourceimpl dyn Error
impl dyn Error
Sourcepub fn sources(&self) -> Source<'_> ⓘ
🔬This is a nightly-only experimental API. (error_iter)
pub fn sources(&self) -> Source<'_> ⓘ
error_iter)Returns an iterator starting with the current error and continuing with
recursively calling Error::source.
If you want to omit the current error and only use its sources,
use skip(1).
Examples
#![feature(error_iter)]
use std::error::Error;
use std::fmt;
#[derive(Debug)]
struct A;
#[derive(Debug)]
struct B(Option<Box<dyn Error + 'static>>);
impl fmt::Display for A {
fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
write!(f, "A")
}
}
impl fmt::Display for B {
fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
write!(f, "B")
}
}
impl Error for A {}
impl Error for B {
fn source(&self) -> Option<&(dyn Error + 'static)> {
self.0.as_ref().map(|e| e.as_ref())
}
}
let b = B(Some(Box::new(A)));
// let err : Box<Error> = b.into(); // or
let err = &b as &dyn Error;
let mut iter = err.sources();
assert_eq!("B".to_string(), iter.next().unwrap().to_string());
assert_eq!("A".to_string(), iter.next().unwrap().to_string());
assert!(iter.next().is_none());
assert!(iter.next().is_none());Trait Implementations
Dyn Compatibility
This trait is dyn compatible.
In older versions of Rust, dyn compatibility was called "object safety".
Implementors
impl !Error for &str
impl Error for !
impl Error for Aborted
std only.impl Error for AccessError
impl Error for AcquireError
impl Error for core::net::parser::AddrParseError
impl Error for ipnet::parser::AddrParseError
impl Error for rustls_pki_types::server_name::AddrParseError
std only.impl Error for AllocError
impl Error for AnyDelimiterCodecError
impl Error for BernoulliError
std only.impl Error for BorrowError
impl Error for BorrowMutError
impl Error for regex_automata::dfa::onepass::BuildError
std only.impl Error for regex_automata::hybrid::error::BuildError
std only.impl Error for regex_automata::meta::error::BuildError
std only.impl Error for regex_automata::nfa::thompson::error::BuildError
std only.impl Error for aho_corasick::util::error::BuildError
std only.impl Error for CacheError
std only.impl Error for Canceled
std only.impl Error for CapacityOverflowError
std only.impl Error for CaseFoldError
std only.impl Error for CharTryFromError
impl Error for DataError
impl Error for DatetimeParseError
std only.impl Error for DecodeError
std only.impl Error for DecodeSliceError
std only.impl Error for DecodeUtf16Error
impl Error for DeserializeError
std only.impl Error for EarlyDataError
std only.impl Error for tokio::time::error::Elapsed
impl Error for tower::timeout::error::Elapsed
impl Error for Empty
std only.impl Error for EncodeError
std only.impl Error for EncodeSliceError
std only.impl Error for EncryptError
std only.impl Error for EnterError
impl Error for rustix::backend::io::errno::Errno
error_in_core or crate feature std only.impl Error for nix::errno::consts::Errno
impl Error for rustix::backend::io::errno::Errno
std only.impl Error for core::fmt::Error
impl Error for cargo_e::prelude::io::Error
impl Error for toml::de::error::Error
std only.impl Error for toml::ser::error::Error
std only.impl Error for serde_core::de::value::Error
std only.impl Error for cargo_metadata::errors::Error
impl Error for semver::parse::Error
std only.impl Error for serde_json::error::Error
impl Error for regex::error::Error
std only.impl Error for regex_syntax::ast::Error
std only.impl Error for regex_syntax::error::Error
std only.impl Error for regex_syntax::hir::Error
std only.impl Error for which::error::Error
impl Error for git2::error::Error
impl Error for icu_collections::codepointtrie::error::Error
impl Error for reqwest::error::Error
impl Error for http::error::Error
impl Error for tokio::time::error::Error
impl Error for hyper_util::client::legacy::client::Error
impl Error for hyper::error::Error
impl Error for h2::error::Error
impl Error for httparse::Error
std only.impl Error for native_tls::Error
impl Error for openssl::error::Error
impl Error for openssl::ssl::error::Error
impl Error for iri_string::normalize::error::Error
std only.impl Error for iri_string::template::error::Error
std only.impl Error for iri_string::validate::Error
std only.impl Error for serde_urlencoded::ser::Error
impl Error for rustls_pki_types::pem::Error
std only.impl Error for rustls::error::Error
std only.impl Error for webpki::error::Error
std only.impl Error for rand::distr::uniform::Error
std only.impl Error for rand::distr::weighted::Error
std only.impl Error for getrandom::error::Error
impl Error for tts::Error
impl Error for speech_dispatcher::Error
impl Error for ctrlc::error::Error
impl Error for ErrorStack
impl Error for Errors
std only.impl Error for ExitStatusError
impl Error for FromBytesUntilNulError
impl Error for FromBytesWithNulError
impl Error for FromOsStrError
impl Error for FromOsStringError
impl Error for FromPathBufError
impl Error for FromPathError
impl Error for FromStrError
impl Error for FromUtf8Error
impl Error for FromUtf16Error
impl Error for FromVecWithNulError
impl Error for core::slice::GetDisjointMutError
impl Error for slab::GetDisjointMutError
std only.impl Error for indexmap::GetDisjointMutError
std only.impl Error for GetTimezoneError
impl Error for GlobError
impl Error for GroupInfoError
std only.impl Error for Infallible
impl Error for IntoStringError
impl Error for InvalidBackoff
impl Error for InvalidDnsNameError
std only.impl Error for InvalidHeaderName
impl Error for InvalidHeaderValue
impl Error for InvalidMethod
impl Error for InvalidNameError
impl Error for InvalidStatusCode
impl Error for InvalidUri
impl Error for InvalidUriParts
impl Error for JoinError
impl Error for JoinPathsError
impl Error for LanguageTagParseError
std only.impl Error for LayoutError
impl Error for LengthDelimitedCodecError
impl Error for LengthLimitError
impl Error for LinesCodecError
impl Error for regex_automata::util::search::MatchError
std only.impl Error for aho_corasick::util::error::MatchError
std only.impl Error for MatchesError
impl Error for MaxSizeReached
impl Error for NonFatalError
impl Error for None
impl Error for NormalizeError
impl Error for NulError
impl Error for OsError
std only.impl Error for OtherError
std only.impl Error for OutOfRange
std only.impl Error for OutOfRangeError
core-error or std only.impl Error for ParseAlphabetError
std only.impl Error for ParseBoolError
impl Error for ParseCharError
impl Error for ParseColorError
impl Error for cargo_platform::error::ParseError
impl Error for bitflags::parser::ParseError
std only.impl Error for chrono::format::ParseError
core-error or std only.impl Error for url::parser::ParseError
std only.impl Error for icu_locale_core::parser::errors::ParseError
impl Error for tinystr::error::ParseError
impl Error for strum::ParseError
std only.impl Error for ParseFloatError
impl Error for ParseIntError
impl Error for log::ParseLevelError
std only.impl Error for tracing_core::metadata::ParseLevelError
std only.impl Error for ParseLevelFilterError
std only.impl Error for ParseMonthError
std only.impl Error for ParseWeekdayError
std only.impl Error for PatternError
impl Error for regex_automata::util::primitives::PatternIDError
std only.impl Error for aho_corasick::util::primitives::PatternIDError
std only.impl Error for PatternSetInsertError
std only.impl Error for PollTimeoutTryFromError
impl Error for PreferencesParseError
impl Error for PrefixLenError
impl Error for cargo_e::prelude::mpsc::RecvError
impl Error for tokio::sync::broadcast::error::RecvError
impl Error for tokio::sync::oneshot::error::RecvError
impl Error for tokio::sync::watch::error::RecvError
impl Error for cargo_e::prelude::mpsc::RecvTimeoutError
impl Error for ReserveError
std only.impl Error for tokio::net::tcp::split_owned::ReuniteError
impl Error for tokio::net::unix::split_owned::ReuniteError
impl Error for RoundingError
std only.impl Error for futures_channel::mpsc::SendError
impl Error for SerializeError
std only.impl Error for SerializerError
std only.impl Error for SetGlobalDefaultError
std only.impl Error for SetLoggerError
std only.impl Error for SmallIndexError
std only.impl Error for SpawnError
std only.impl Error for StartError
std only.impl Error for regex_automata::util::primitives::StateIDError
std only.impl Error for aho_corasick::util::primitives::StateIDError
std only.impl Error for StrSimError
impl Error for StripPrefixError
impl Error for SystemTimeError
impl Error for ToCompactStringError
std only.impl Error for ToStrError
impl Error for TryAcquireError
impl Error for TryCurrentError
impl Error for TryFromCharError
impl Error for TryFromFloatSecsError
impl Error for TryFromIntError
impl Error for TryFromSliceError
impl Error for TryGetError
std only.impl Error for cargo_e::prelude::fs::TryLockError
impl Error for tokio::sync::mutex::TryLockError
impl Error for cargo_e::prelude::mpsc::TryRecvError
impl Error for futures_channel::mpsc::TryRecvError
impl Error for tokio::sync::broadcast::error::TryRecvError
impl Error for tokio::sync::mpsc::error::TryRecvError
impl Error for tokio::sync::oneshot::error::TryRecvError
impl Error for alloc::collections::TryReserveError
impl Error for indexmap::TryReserveError
std only.impl Error for UleError
impl Error for UnicodeWordBoundaryError
std only.impl Error for UnicodeWordError
std only.impl Error for UnorderedKeyError
impl Error for UnsupportedOperationError
std only.impl Error for Utf8Error
impl Error for VarError
impl Error for VerifierBuilderError
std only.impl Error for WriterPanicked
impl Error for X509VerifyResult
impl Error for ZeroTrieBuildError
impl<'a, T> Error for &'a T
impl<A, S, V> Error for ConvertError<A, S, V>
zerocopy_core_error_1_81_0 or crate feature std only.impl<E> Error for Box<E>where
E: Error,
impl<F> Error for clap_builder::error::Error<F>where
F: ErrorFormatter,
impl<I> Error for ExactlyOneError<I>
use_std only.impl<L, R> Error for Either<L, R>
std only.Either implements Error if both L and R implement it.
Requires crate feature "std"
impl<S> Error for native_tls::HandshakeError<S>
impl<S> Error for openssl::ssl::error::HandshakeError<S>where
S: Debug,
impl<Src, Dst> Error for AlignmentError<Src, Dst>
zerocopy_core_error_1_81_0 or crate feature std only.impl<Src, Dst> Error for SizeError<Src, Dst>
zerocopy_core_error_1_81_0 or crate feature std only.impl<Src, Dst> Error for ValidityError<Src, Dst>
zerocopy_core_error_1_81_0 or crate feature std only.impl<T, E> Error for TryChunksError<T, E>
std only.impl<T, E> Error for TryReadyChunksError<T, E>
std only.impl<T, Item> Error for futures_util::stream::stream::split::ReuniteError<T, Item>where
T: Any,
std only.impl<T> Error for Arc<T>
impl<T> Error for AsyncFdTryNewError<T>
impl<T> Error for iri_string::template::error::CreationError<T>where
T: Debug,
std only.impl<T> Error for iri_string::types::generic::error::CreationError<T>where
T: Debug,
std only.impl<T> Error for PoisonError<T>
impl<T> Error for PollSendError<T>where
T: Debug,
impl<T> Error for std::sync::oneshot::RecvTimeoutError<T>
impl<T> Error for futures_util::io::split::ReuniteError<T>where
T: Any,
std only.impl<T> Error for cargo_e::prelude::mpsc::SendError<T>
impl<T> Error for tokio::sync::mpsc::error::SendError<T>
impl<T> Error for tokio::sync::watch::error::SendError<T>
impl<T> Error for tokio::sync::broadcast::error::SendError<T>where
T: Debug,
impl<T> Error for std::sync::mpmc::error::SendTimeoutError<T>
impl<T> Error for tokio::sync::mpsc::error::SendTimeoutError<T>
time only.